@@ -1,10 +1,30 @@
 /*
- * Copyright (c) 1991-2013 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2016 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2013 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2016 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
+#ifdef HAVE_LIBREADLINE
+# if defined(HAVE_READLINE_READLINE_H)
+#  include <readline/readline.h>
+# elif defined(HAVE_READLINE_H)
+#  include <readline.h>
+# else
+extern char *readline();
+# endif
+#endif	/* HAVE_LIBREADLINE */
+
+#ifdef HAVE_LIBREADLINE_HISTORY
+# if defined(HAVE_READLINE_HISTORY_H)
+#  include <readline/history.h>
+# elif defined(HAVE_HISTORY_H)
+#  include <history.h>
+# else
+extern void add_history();
+# endif
+#endif	/* HAVE_LIBREADLINE_HISTORY */
+
 typedef struct __wtoken__ {
   WORD_ID wid;
   struct __wtoken__ *next;
