@@ -111,7 +111,7 @@
  * @author Akinobu Lee
  * @date   Tue Aug 23 11:44:14 2005
  *
- * $Revision: 1.13 $
+ * $Revision: 1.14 $
  * 
  */
 /*
@@ -173,8 +173,8 @@ init_param(MFCCCalc *mfcc)
   if (para->cmn) mfcc->param->header.samptype |= F_CEPNORM;
   
   mfcc->param->header.wshift = para->smp_period * para->frameshift;
-  mfcc->param->header.sampsize = para->veclen * sizeof(VECT); /* not compressed */
-  mfcc->param->veclen = para->veclen;
+  mfcc->param->header.sampsize = para->veclen * mfcc->splice * sizeof(VECT); /* not compressed */
+  mfcc->param->veclen = para->veclen * mfcc->splice;
   
   /* ǧ/λ˥åȤѿ:
      param->parvec (ѥ᡼٥ȥ)
@@ -220,12 +220,10 @@ boolean
 RealTimeInit(Recog *recog)
 {
   Value *para;
-  Jconf *jconf;
   RealBeam *r;
   MFCCCalc *mfcc;
 
 
-  jconf = recog->jconf;
   r = &(recog->real);
 
   /* ե졼Ĺϻֿ׻ */
@@ -251,7 +249,6 @@ RealTimeInit(Recog *recog)
       mfcc->wrk->ss_floor = mfcc->frontend.ss_floor;
     }
   }
-
   for(mfcc = recog->mfcclist; mfcc; mfcc = mfcc->next) {
   
     para = mfcc->para;
@@ -266,15 +263,21 @@ RealTimeInit(Recog *recog)
     /* ǥ륿׻ΤΥꥢ */
     /* allocate work area for the delta computation */
     mfcc->tmpmfcc = (VECT *)mymalloc(sizeof(VECT) * para->vecbuflen);
+    /* splice buffer */
+    if (mfcc->splice > 1) {
+      mfcc->splicedmfcc = (VECT *)mymalloc(sizeof(VECT) * para->veclen * mfcc->splice);
+      mfcc->splicedlen = 0;
+    }
     /* MAP-CMN ѤνץȥʿѤɤ߹ǽ */
     /* Initialize the initial cepstral mean data from file for MAP-CMN */
-    if (para->cmn || para->cvn) mfcc->cmn.wrk = CMN_realtime_new(para, mfcc->cmn.map_weight);
+    if (para->cmn || para->cvn) mfcc->cmn.wrk = CMN_realtime_new(para, mfcc->cmn.map_weight, mfcc->cmn.map_cmn);
     /* -cmnload , CMNѤΥץȥʿѤνͤե뤫ɤ߹ */
     /* if "-cmnload", load initial cepstral mean data from file for CMN */
     if (mfcc->cmn.load_filename) {
       if (para->cmn) {
 	if ((mfcc->cmn.loaded = CMN_load_from_file(mfcc->cmn.wrk, mfcc->cmn.load_filename))== FALSE) {
-	  jlog("WARNING: failed to read initial cepstral mean from \"%s\", do flat start\n", mfcc->cmn.load_filename);
+	  jlog("ERROR: failed to read initial cepstral mean from \"%s\", do flat start\n", mfcc->cmn.load_filename);
+	  return FALSE;
 	}
       } else {
 	jlog("WARNING: CMN not required on AM, file \"%s\" ignored\n", mfcc->cmn.load_filename);
@@ -321,9 +324,6 @@ reset_mfcc(Recog *recog)
 {
   Value *para;
   MFCCCalc *mfcc;
-  RealBeam *r;
-
-  r = &(recog->real);
 
   /* ħХ⥸塼 */
   /* initialize parameter extraction module */
@@ -338,6 +338,7 @@ reset_mfcc(Recog *recog)
     /* set the delta cycle buffer */
     if (para->delta) WMP_deltabuf_prepare(mfcc->db);
     if (para->acc) WMP_deltabuf_prepare(mfcc->ab);
+    if (mfcc->splice > 1) mfcc->splicedlen = 0;
   }
 
 }
@@ -405,6 +406,7 @@ RealTimePipeLinePrepare(Recog *recog)
   /* check type coherence between param and hmminfo here */
   if (recog->jconf->input.paramtype_check_flag) {
     for(am=recog->amlist;am;am=am->next) {
+      if (am->config->dnn.enabled) continue;
       if (!check_param_coherence(am->hmminfo, am->mfcc->param)) {
 	jlog("ERROR: input parameter type does not match AM\n");
 	return FALSE;
@@ -440,6 +442,24 @@ RealTimePipeLinePrepare(Recog *recog)
   return TRUE;
 }
 
+/* splice */
+static boolean
+splice_mfcc(MFCCCalc *mfcc)
+{
+  if (mfcc->splicedlen >= mfcc->splice) {
+    memmove(mfcc->splicedmfcc, &(mfcc->splicedmfcc[mfcc->para->veclen]), sizeof(VECT) * (mfcc->splice - 1) * mfcc->para->veclen);
+    mfcc->splicedlen --;
+  }
+  memcpy(&(mfcc->splicedmfcc[mfcc->para->veclen * mfcc->splicedlen]), mfcc->tmpmfcc, sizeof(VECT) * mfcc->para->veclen);
+  mfcc->splicedlen++;
+  if (mfcc->splicedlen < mfcc->splice) {
+    /* if ret == FALSE, there is no available frame.  So just wait for
+       next input */
+    return FALSE;
+  }
+  return TRUE;
+}
+
 /** 
  * <JA>
  * @brief  ȷѥ᡼٥ȥ׻.
@@ -573,6 +593,11 @@ RealTimeMFCC(MFCCCalc *mfcc, SP16 *window, int windowlen)
   /* perform CMN */
   if (para->cmn || para->cvn) CMN_realtime(mfcc->cmn.wrk, tmpmfcc);
 
+  /* splice */
+  if (mfcc->splice > 1) {
+    return(splice_mfcc(mfcc));
+  }
+
   return TRUE;
 }
 
@@ -790,6 +815,7 @@ RealTimePipeLine(SP16 *Speech, int nowlen, Recog *recog) /* Speech[0...nowlen] =
   int i, now, ret;
   MFCCCalc *mfcc;
   RealBeam *r;
+  VECT *mfccvec;
 
   r = &(recog->real);
 
@@ -843,9 +869,14 @@ RealTimePipeLine(SP16 *Speech, int nowlen, Recog *recog) /* Speech[0...nowlen] =
       /* calculate a parameter vector from current waveform windows
 	 and store to r->tmpmfcc */
       if ((*(recog->calc_vector))(mfcc, r->window, r->windowlen)) {
+	if (mfcc->splice > 1) {
+	  mfccvec = mfcc->splicedmfcc;
+	} else {
+	  mfccvec = mfcc->tmpmfcc;
+	}
 #ifdef ENABLE_PLUGIN
 	/* call post-process plugin if exist */
-	plugin_exec_vector_postprocess(mfcc->tmpmfcc, mfcc->param->veclen, mfcc->f);
+	plugin_exec_vector_postprocess(mfccvec, mfcc->param->veclen, mfcc->f);
 #endif
 	/* MFCCϿ */
   	mfcc->valid = TRUE;
@@ -854,7 +885,7 @@ RealTimePipeLine(SP16 *Speech, int nowlen, Recog *recog) /* Speech[0...nowlen] =
 	  jlog("ERROR: failed to allocate memory for incoming MFCC vectors\n");
 	  return -1;
 	}
-	memcpy(mfcc->param->parvec[mfcc->f], mfcc->tmpmfcc, sizeof(VECT) * mfcc->param->veclen);
+	memcpy(mfcc->param->parvec[mfcc->f], mfccvec, sizeof(VECT) * mfcc->param->veclen);
 #ifdef RDEBUG
 	printf("DeltaBuf: %02d: got frame %d\n", mfcc->id, mfcc->f);
 #endif
@@ -1118,6 +1149,7 @@ RealTimeParam(Recog *recog)
 #ifdef RDEBUG
   int i;
 #endif
+  VECT *mfccvec;
 
   r = &(recog->real);
 
@@ -1255,12 +1287,21 @@ RealTimeParam(Recog *recog)
       }
       /* a new frame has been obtained from delta buffer to tmpmfcc */
       if(para->cmn || para->cvn) CMN_realtime(mfcc->cmn.wrk, mfcc->tmpmfcc);
+      /* splice */
+      if (mfcc->splice > 1) {
+	if (splice_mfcc(mfcc) == FALSE) {
+	  continue;
+	}
+	mfccvec = mfcc->splicedmfcc;
+      } else {
+	mfccvec = mfcc->tmpmfcc;
+      }
       if (param_alloc(mfcc->param, mfcc->f + 1, mfcc->param->veclen) == FALSE) {
 	jlog("ERROR: failed to allocate memory for incoming MFCC vectors\n");
 	return FALSE;
       }
       /* store to mfcc->f */
-      memcpy(mfcc->param->parvec[mfcc->f], mfcc->tmpmfcc, sizeof(VECT) * mfcc->param->veclen);
+      memcpy(mfcc->param->parvec[mfcc->f], mfccvec, sizeof(VECT) * mfcc->param->veclen);
 #ifdef ENABLE_PLUGIN
       /* call postprocess plugin if any */
       plugin_exec_vector_postprocess(mfcc->param->parvec[mfcc->f], mfcc->param->veclen, mfcc->f);
@@ -1367,10 +1408,8 @@ RealTimeCMNUpdate(MFCCCalc *mfcc, Recog *recog)
 {
   boolean cmn_update_p;
   Value *para;
-  Jconf *jconf;
   RecogProcess *r;
 
-  jconf = recog->jconf;
   para = mfcc->para;
   
   /* update CMN vector for next speech */
@@ -1450,10 +1489,6 @@ RealTimeTerminate(Recog *recog)
 void
 realbeam_free(Recog *recog)
 {
-  RealBeam *r;
-
-  r = &(recog->real);
-
   if (recog->real.window) {
     free(recog->real.window);
     recog->real.window = NULL;
