@@ -21,13 +21,13 @@
  * @author Akinobu LEE
  * @date   Sat Feb 12 12:30:40 2005
  *
- * $Revision: 1.9 $
+ * $Revision: 1.10 $
  * 
  */
 /*
- * Copyright (c) 1991-2012 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2013 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2012 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2013 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -88,6 +88,8 @@ void *mymalloc(size_t size);
 void *mymalloc_big(size_t elsize, size_t nelem);
 void *myrealloc(void *, size_t);
 void *mycalloc(size_t, size_t);
+void *mymalloc_aligned(size_t size, size_t align);
+void myfree_aligned(void *ptr);
 
 /* endian.c */
 void swap_sample_bytes(SP16 *buf, int len);
