@@ -17,13 +17,13 @@
  * @author Akinobu LEE
  * @date   Thu Feb 17 16:27:03 2005
  *
- * $Revision: 1.7 $
+ * $Revision: 1.8 $
  * 
  */
 /*
- * Copyright (c) 1991-2012 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2013 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2012 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2013 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -133,3 +133,33 @@ mycalloc(size_t nelem, size_t elsize)
   return p;
 }
 
+/* MMDFiles_alignedmalloc: aligned malloc*/
+void *mymalloc_aligned(size_t size, size_t align)
+{
+   void *ptr;
+#if defined(_MSC_VER)
+   ptr = _aligned_malloc(size, align);
+#elif defined(__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
+   ptr = aligned_alloc(align, size);
+#elif defined(_POSIX_VERSION) && _POSIX_VERSION >= 200112L
+   if (posix_memalign(&ptr, align, size) != 0)
+      ptr = NULL;
+#else
+   ptr = malloc(size);
+#endif // defined is defined, use the Windows stuff.
+   return ptr;
+}
+
+/* MMDFiles_alignedfree: free aligned malloc */
+void myfree_aligned(void *ptr)
+{
+#if defined(_MSC_VER)
+   _aligned_free(ptr);
+#elif defined(__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
+   free(ptr);
+#elif defined(_POSIX_VERSION) && _POSIX_VERSION >= 200112L
+   free(ptr);
+#else
+   free(ptr);
+#endif // defined is defined, use the Windows stuff.
+}
